clear all; close all;

%switching off annoying warnings of ode45
warning('off','MATLAB:ode45:IntegrationTolNotMet');


%% Select an example 

example='4a';

switch example
    %Harald's example (lpv plant + gain scheduled PI controller)
    case '1a', 
        lpvsys=@lpvsys_harald; 
        ptrajectory=@traj_pwcder; 
        tdata=[1 0 -1 0 1 0]; gU=2.97; hRange=[12 20]; 
        filename='harald_pwl_'; fileidx=0; MaxIter=50;
    case '1b', 
        lpvsys=@lpvsys_harald; 
        ptrajectory=@traj_sinusoids; 
        msk=[0];
        fmp=[1];
        tdata={fmp,msk}; gU=2.97; hRange=[12 20]; 
        filename='harald_sin_'; fileidx=0; MaxIter=50;
    case '1c', 
        lpvsys=@lpvsys_harald; 
        ptrajectory=@traj_sinusoids; 
        msk=[1 1 1 1 1 1 1 1 1];
        fmp=[1 2 3 4 5 6 7 8 9];
        tdata={fmp,msk}; gU=2.97; hRange=[12 20]; 
        filename='harald_msin_'; fileidx=0; MaxIter=50;
     
    
        
    %Two scaled, interconnected LTI systems    
    case '2a',
        ratelims=[0.1 0.4 0.7 1 1.3 1.6 2];
        g_ubnds=[0.1087 0.3342 0.4805 0.5766 0.6435 0.6924 0.7403];
        r=3;
        lpvsys=@(p) lpvsys_catws(p,ratelims(r)); 
        ptrajectory=@traj_pwcder;
        tdata=ratelims(r)*[1 0 -1 0 1 0]; hRange=[0.5 20]; gU=g_ubnds(r); 
        filename=['catws_' sprintf('%02d',10*ratelims(r)) '_']; fileidx=0;
        MaxIter=100;
    
     case '3a', 
        lpvsys=@lpvsys_process; lpvsys('init'); 
        ptrajectory=@traj_pwcder; 
        tdata=2000*[1 0 -1 0 1 0 -1]; gU=1.68; hRange=[.1 10]; 
        filename='process_'; fileidx=0; MaxIter=50;
     case '3b', 
        lpvsys=@lpvsys_process; lpvsys('init'); 
        ptrajectory=@traj_sinusoids; 
        msk=[1 1 1 1 1 1 1 1 1];
        fmp=[1 2 3 4 5 6 7 8 9];
        tdata={fmp,msk}; gU=1.68; hRange=[.1 10]; 
        filename='process'; fileidx=0; MaxIter=50;
    
     %Rotated LTI systems     
     case '4a', 
        lpvsys=@lpvsys_rotated;  
        ptrajectory=@traj_pwcder; 
        tdata=0.1*[1 0 -1 0 1 0]; gU=3.3; hRange=[1 5]; 
        filename='rotated_'; fileidx=0; MaxIter=50;
     
end;

directory='./matfiles/';
if ~exist('psopts','var'), psopts=[]; end;

while 1,
    
%% Loading the initial value
if fileidx>0,
    file=load([directory filename sprintf('%02d',fileidx)],'c');
    c0=file.c;
    
    i=1;
    while i>0,
        str=[directory filename sprintf('%02d',fileidx+i) '.mat']; 
        if exist(str,'file'), 
            delete(str); i=i+1;
        else
            i=0;
        end;
    end;
         
else
    delete([directory filename '*.mat']);
    if ~exist('c0','var'), c0=[]; end;
end;
fileidx=fileidx+1;    




%% Optimization (maximizing nu)
[nu_opt,c_opt,c_init]=max_nu(lpvsys,ptrajectory,tdata,gU,'patternsearch',c0,hRange,MaxIter,psopts);
c=c_opt;
h=ptrajectory([],c,tdata,-1); w=2*pi/h;


%% Computing gamma
verbose=1;
[gL,gL_high]=periodicL2gain(@(t) persys(t,lpvsys,ptrajectory,c,tdata),verbose)


%% Plotting the trajectory
figure; 
Nt = 2000; h=2*pi/w;
t = linspace(0,3*h,Nt);
[p,dp]=ptrajectory(t,c,tdata);
subplot(2,1,1); plot(t,p,'LineWidth',2); ylabel('\rho(t)');
subplot(2,1,2); plot(t,dp,'LineWidth',2); ylabel('d\rho(t)/dt');
hold on;
grid on; axis tight;
xlabel('t'); 


%% Saving the result
save([directory filename sprintf('%02d',fileidx)]);
reply=input('Continue?','s');
if reply~='y' && reply~='Y', break; end;


end;










