% Computing an upper bound on the L2 norm by finding a maximal gamma (g) and a storage function V that solves 
% the dissipativity relation Vdot-g^2w^Tw+z^Tz<0.


%% Select an example
example=4;

switch example
    case 1,
        lpvsys=@lpvsys_harald;
        Pbase=@(rho) [1 rho rho^2 rho^3 rho^4 rho^5 1/rho 1/rho^2  rho^6 1/rho^3];
        Pbase_der=@(rho,nu) [0 nu 2*rho*nu 3*rho^2*nu  4*rho^3*nu 5*rho^4*nu -nu/rho^2 -2*rho*nu/rho^4 6*rho^5*nu ...
            -3*rho^2*nu/rho^6];
        M=100; glb=2.9; 
        %solver='sdpt3';  maxiter=200; 
        solver='mosek';
    case 2,
        lpvsys=@(p) lpvsys_catws(p,2);
        Pbase=@(rho) [1 rho rho^2 rho^3 rho^4 rho^5 rho^6 rho^7 rho^8 rho^9 rho^10];
        Pbase_der=@(rho,nu) [0 nu 2*rho*nu 3*rho^2*nu 4*rho^3*nu 5*rho^4*nu 6*rho^5*nu ...
                                7*rho^6*nu 8*rho^7*nu 9*rho^8*nu 10*rho^9*nu];
        M=100; glb=0.738; 
        %solver='sdpt3';  maxiter=50;   
        solver='mosek';
    case 3,
        lpvsys=@lpvsys_process; lpvsys('init');
        Pbase=@(rho) [1 rho^2 rho^3 rho^4 rho^5 rho^6 rho^7 rho^8 rho^9 rho^10 rho^11 rho^12];%  rho^6 1/rho^3];
        Pbase_der=@(rho,nu) [0 2*rho*nu 3*rho^2*nu  4*rho^3*nu 5*rho^4*nu 6*rho^5*nu 7*rho^6*nu 8*rho^7*nu ...
            9*rho^8*nu 10*rho^9*nu 11*rho^10*nu 12*rho^11*nu];
            %-3*rho^2*nu/rho^6];
        M=20; glb=1.38; 
        solver='sdpt3';  maxiter=200; 
        solver='mosek';
    case 4,
        lpvsys=@lpvsys_rotated; 
        Pbase=@(rho) [1 rho rho^2 rho^3 rho^4 rho^5 rho^6 rho^7];% rho^8 rho^9 rho^10 rho^11 rho^12];%  rho^6 1/rho^3];
        Pbase_der=@(rho,nu) [0 nu 2*rho*nu 3*rho^2*nu  4*rho^3*nu 5*rho^4*nu 6*rho^5*nu 7*rho^6*nu];% 8*rho^7*nu ...
            %9*rho^8*nu 10*rho^9*nu 11*rho^10*nu 12*rho^11*nu];
            %-3*rho^2*nu/rho^6];
        M=50; glb=3.1; 
        solver='sdpt3';  maxiter=200; 
        solver='mosek'
end;



%% The alorithm starts here
[nx,nw,nz,rho_bnds,nu_bnds]=lpvsys([]);
Pbf=Pbase(rho_bnds(:,1)); 
nofbases=length(Pbf);
if nofbases==1, N=1; else N=2; end; %here it is assumed that if the number of bases==1, then
                                    %this one basis function is the constant 1.
[rho_grid,nu_grid]=mgrid(rho_bnds,M,nu_bnds,N);
sizeofgrd=size(rho_grid,1);

%Constructing the LMIs for the Bounded Real Lemma-like conditions
posdef=@(LMI) (LMI>=1e-8*eye(size(LMI)));
negdef=@(LMI) (LMI<=-1e-8*eye(size(LMI)));

P=cell(1,nofbases); Cons=[];
for i=1:nofbases
    P{i}=sdpvar(nx);
end;

g2=sdpvar(1); 
Cons=(g2>=glb^2); 
Obj=g2;

% % % g2=glb^2;
% % % Cons=[];
% % % Obj=[];
    
prgrs=0; prevprgrs=-1; fprintf('Building up the LMIs......');
for k=1:sizeofgrd
    prgrs=fix(100*k/sizeofgrd); 
    if prgrs>prevprgrs, fprintf('\b\b\b\b%3d%%',prgrs); end; 
    prevprgrs=prgrs;
    
    
    rho_k=rho_grid(k,:);
    nu_k =nu_grid(k,:);
    
    [Ak,Bk,Ck,Dk]=lpvsys(rho_k);
    Pbf=Pbase(rho_k);
    Pbf_der=Pbase_der(rho_k,nu_k);
    
    
    Pk=0; Pkdot=0;
    for i=1:nofbases
        Pk=Pk+Pbf(i)*P{i};
        Pkdot=Pkdot+Pbf_der(i)*P{i};
    end;
    Cons=Cons+posdef(Pk);
    Cons=Cons+negdef([Pkdot+Pk*Ak+Ak'*Pk+Ck'*Ck Pk*Bk+Ck'*Dk; Bk'*Pk+Dk'*Ck Dk'*Dk-g2*eye(nw)]);
end;


fprintf('\nCalling the solver......');

switch solver
    case 'sdpt3',
        options=sdpsettings('verbose',1,'solver','sdpt3','sdpt3.maxit',maxiter);
    case 'sedumi',
        options=sdpsettings('verbose',1,'solver','sedumi','sedumi.maxiter',maxiter);        
    case 'mosek'
        options=sdpsettings('verbose',1,'solver','mosek');
end;
        
        
sol=solvesdp(Cons,Obj,options);
if sol.problem ~= 0
    display('Hmm, something went wrong, but we check the results.');
    sol.info
    %yalmiperror(sol.problem)   
end;

g2=double(g2);
gam=sqrt(g2)
for i=1:nofbases
    P{i}=double(P{i});
end;
    
%Checking the result
for k=1:sizeofgrd
    rho_k=rho_grid(k,:);
    nu_k =nu_grid(k,:);
    
    [Ak,Bk,Ck,Dk]=lpvsys(rho_k);
    Pbf=Pbase(rho_k);
    Pbf_der=Pbase_der(rho_k,nu_k);
       
    Pk=0; Pkdot=0;
    for i=1:nofbases
        Pk=Pk+Pbf(i)*P{i};
        Pkdot=Pkdot+Pbf_der(i)*P{i};
    end;
    %eig([Pkdot+Pk*Ak+Ak'*Pk+Ck'*Ck Pk*Bk+Ck'*Dk; Bk'*Pk+Dk'*Ck Dk'*Dk-g2*eye(nw)])
    if any(eig(Pk,'nobalance')<1e-8), error('P is not pos. def'); end;
    if any(eig([Pkdot+Pk*Ak+Ak'*Pk+Ck'*Ck Pk*Bk+Ck'*Dk; Bk'*Pk+Dk'*Ck Dk'*Dk-g2*eye(nw)],'nobalance')>-1e-8) 
        error('Dissip. ineq. is not neg. def.'); end;
end;

    
    


    


    
    
    
    
    
    
    
    





    