function [nu_opt,x_opt]=max_nu(lpvsys,ptrajectory,fmp,msk,g,solver,c0,hRange,cfixed,MaxIter)
% Maximizing nu, where nu is the induced L2 norm of the discrete-time LTI
% system associated with the periodic LTV system obtained by evaluating the 
% LPV system over a periodic scheduling trajectory.


[~,~,~,Range,Rate]=lpvsys([]);

info=functions(ptrajectory);
switch info.function
    case 'traj_pwcder2p'
        i=str2double(solver(end)); 
        if ~exist('cfixed','var'); cfixed=[]; end; 
        [Alp,blp,Aeq,beq]=ptrajectory([],[],fmp,msk,i,Range,hRange,cfixed);
        L=null(Aeq); cstar=Aeq\beq; blp=blp-Alp*cstar; Alp=Alp*L; 
        P=Polyhedron(Alp,blp); dimP=size(P.H,2)-1;
        if dimP<20, P.minHRep(); end;
        if dimP<4, plot(P,'Wire',true,'LineWidth',2); hold on; drawnow; end;
        if isempty(c0), c0=cprnd(1,P.H(:,1:end-1),P.H(:,end)); c0=c0'; 
        else c0=L\(c0-cstar); end
        B=P.outerApprox;
        x_U=B.H(1:dimP,end);
        x_L=-B.H(dimP+1:end,end);
        b_L=-inf*ones(size(P.H,1),1);
        b_U=P.H(:,end);
        A=P.H(:,1:end-1);
  
        
        switch solver
            case 'patternsearch_1'
                opts = psoptimset(psoptimset(@patternsearch),'Display','iter',...
                        'MaxIter',MaxIter,'TolFun',1e-5,'TolMesh',1e-5,'TolX',1e-5);
                [c_opt, nu_opt] = patternsearch(@(c) -obj_nu([L*c+cstar;cfixed],g,lpvsys,ptrajectory,fmp,msk),...
                                  c0,A,b_U,[],[],x_L,x_U,[],opts);
                nu_opt=-nu_opt;
                c_opt=[L*c_opt+cstar;cfixed];
                x_opt=c_opt;
                
            case 'patternsearch_2'
                opts = psoptimset(psoptimset(@patternsearch),'Display','iter',...
                        'MaxIter',MaxIter,'TolFun',1e-5,'TolMesh',1e-5,'TolX',1e-5);
                [c_opt, nu_opt] = patternsearch(@(c) -obj_nu([cfixed;L*c+cstar],g,lpvsys,ptrajectory,fmp,msk),...
                                  c0,A,b_U,[],[],x_L,x_U,[],opts);
                nu_opt=-nu_opt;
                c_opt=[cfixed;L*c_opt+cstar];
                x_opt=c_opt;
            case 'patternsearch_3'
                opts = psoptimset(psoptimset(@patternsearch),'Display','iter',...
                        'MaxIter',MaxIter,'TolFun',1e-5,'TolMesh',1e-5,'TolX',1e-5);
                [c_opt, nu_opt] = patternsearch(@(c) -obj_nu(L*c+cstar,g,lpvsys,ptrajectory,fmp,msk),...
                                  c0,A,b_U,[],[],x_L,x_U,[],opts);
                nu_opt=-nu_opt;
                c_opt=L*c_opt+cstar;
                x_opt=c_opt;
            
        end;
    
        
        
    otherwise
        error('Ooops!');
         
end;


        



